@@ -6,13 +6,14 @@
  * SPDX-License-Identifier: MIT
  */
 
-#define G_LOG_DOMAIN "wp-properties"
-
 #include "properties.h"
+#include "log.h"
 
 #include <errno.h>
 #include <pipewire/properties.h>
 
+WP_DEFINE_LOCAL_LOG_TOPIC ("wp-properties")
+
 /*! \defgroup wpproperties WpProperties */
 /*!
  * \struct WpProperties
@@ -127,8 +128,8 @@ wp_properties_new_valist (const gchar * key, va_list args)
  * be parsed from the given string
  *
  * \ingroup wpproperties
- * \param str a string containing a whitespace separated list of key=value pairs
- *    (ex. "key1=value1 key2=value2")
+ * \param str a string containing either a whitespace separated list of key=value
+ *    pairs (ex. "key1=value1 key2=value2") or a JSON object (ex. '{"key1":"value1"}')
  * \returns (transfer full): the newly constructed properties set
  */
 WpProperties *
@@ -145,6 +146,26 @@ wp_properties_new_string (const gchar * str)
   return self;
 }
 
+/*!
+ * \brief Constructs a new properties set that contains the properties that can
+ * be parsed from the given JSON object
+ *
+ * \ingroup wpproperties
+ * \param json a JSON object
+ * \returns (transfer full): the newly constructed properties set
+ */
+WpProperties *
+wp_properties_new_json (const WpSpaJson * json)
+{
+  WpProperties * self;
+
+  g_return_val_if_fail (json != NULL, NULL);
+
+  self = wp_properties_new_empty ();
+  wp_properties_update_from_json (self, json);
+  return self;
+}
+
 /*!
  * \brief Constructs a new WpProperties that wraps the given \a props structure,
  * allowing reading properties on that \a props structure through
@@ -182,7 +203,7 @@ wp_properties_new_wrap (const struct pw_properties * props)
  * allowing reading & writing properties on that \a props structure through
  * the WpProperties API.
  *
- * In constrast with wp_properties_new_wrap(), this function assumes ownership
+ * In contrast with wp_properties_new_wrap(), this function assumes ownership
  * of the \a props structure, so it will try to free \a props when it is destroyed.
  *
  * \ingroup wpproperties
@@ -397,6 +418,28 @@ wp_properties_update_from_dict (WpProperties * self,
   return pw_properties_update (self->props, dict);
 }
 
+/*!
+ * \brief Updates (adds new or modifies existing) properties in \a self,
+ * using the given \a json as a source.
+ *
+ * Any properties that are not contained in \a json are left untouched.
+ *
+ * \ingroup wpproperties
+ * \param self a properties object
+ * \param json a JSON object that contains properties to update
+ * \returns the number of properties that were changed
+ */
+gint
+wp_properties_update_from_json (WpProperties * self, const WpSpaJson * json)
+{
+  g_return_val_if_fail (self != NULL, -EINVAL);
+  g_return_val_if_fail (!(self->flags & FLAG_IS_DICT), -EINVAL);
+  g_return_val_if_fail (!(self->flags & FLAG_NO_OWNERSHIP), -EINVAL);
+
+  return pw_properties_update_string (self->props, wp_spa_json_get_data (json),
+      wp_spa_json_get_size (json));
+}
+
 /*!
  * \brief Adds new properties in \a self, using the given \a props as a source.
  *
@@ -894,40 +937,6 @@ wp_properties_new_iterator (WpProperties * self)
   return g_steal_pointer (&it);
 }
 
-/*!
- * \brief Gets the key from a properties iterator item
- *
- * \ingroup wpproperties
- * \param item a GValue that was returned from the WpIterator of
- *   wp_properties_new_iterator()
- * \returns (transfer none): the property key of the \a item
- * \deprecated Use wp_properties_item_get_key() instead
- */
-const gchar *
-wp_properties_iterator_item_get_key (const GValue * item)
-{
-  WpPropertiesItem *pi = g_value_get_boxed (item);
-  g_return_val_if_fail (pi != NULL, NULL);
-  return wp_properties_item_get_key (pi);
-}
-
-/*!
- * \brief Gets the value from a properties iterator item
- *
- * \ingroup wpproperties
- * \param item a GValue that was returned from the WpIterator of
- *   wp_properties_new_iterator()
- * \returns (transfer none): the property value of the \a item
- * \deprecated Use wp_properties_item_get_value() instead
- */
-const gchar *
-wp_properties_iterator_item_get_value (const GValue * item)
-{
-  WpPropertiesItem *pi = g_value_get_boxed (item);
-  g_return_val_if_fail (pi != NULL, NULL);
-  return wp_properties_item_get_value (pi);
-}
-
 /*!
  * \brief Gets the number of properties contained in this object
  * \ingroup wpproperties
@@ -1027,7 +1036,7 @@ wp_properties_unref_and_take_pw_properties (WpProperties * self)
  * \ingroup wpproperties
  * \param self a properties object
  * \param other a set of properties to match
- * \returns TRUE if all matches were successfull, FALSE if at least one
+ * \returns TRUE if all matches were successful, FALSE if at least one
  *   property value did not match
  */
 gboolean
