@@ -9,15 +9,18 @@
 #ifndef __WIREPLUMBER_WP_H__
 #define __WIREPLUMBER_WP_H__
 
+#include "base-dirs.h"
 #include "client.h"
 #include "component-loader.h"
+#include "conf.h"
 #include "core.h"
-#include "dbus.h"
 #include "device.h"
-#include "endpoint.h"
 #include "error.h"
+#include "event-dispatcher.h"
+#include "event-hook.h"
 #include "global-proxy.h"
 #include "iterator.h"
+#include "json-utils.h"
 #include "link.h"
 #include "log.h"
 #include "metadata.h"
@@ -28,6 +31,7 @@
 #include "object.h"
 #include "plugin.h"
 #include "port.h"
+#include "proc-utils.h"
 #include "properties.h"
 #include "proxy.h"
 #include "proxy-interfaces.h"
@@ -42,6 +46,7 @@
 #include "wpenums.h"
 #include "wpversion.h"
 #include "factory.h"
+#include "settings.h"
 
 G_BEGIN_DECLS
 
@@ -67,36 +72,10 @@ WP_API
 void wp_init (WpInitFlags flags);
 
 WP_API
-const gchar * wp_get_module_dir (void);
+const char * wp_get_library_version (void);
 
 WP_API
-G_DEPRECATED_FOR (wp_find_file)
-const gchar * wp_get_config_dir (void);
-
-WP_API
-G_DEPRECATED_FOR (wp_find_file)
-const gchar * wp_get_data_dir (void);
-
-/*!
- * \brief Flags to specify lookup directories
- * \ingroup wp
- */
-typedef enum { /*< flags >*/
-  WP_LOOKUP_DIR_ENV_CONFIG = (1 << 0),       /*!< $WIREPLUMBER_CONFIG_DIR */
-  WP_LOOKUP_DIR_ENV_DATA = (1 << 1),         /*!< $WIREPLUMBER_DATA_DIR */
-
-  WP_LOOKUP_DIR_XDG_CONFIG_HOME = (1 << 10), /*!< XDG_CONFIG_HOME/wireplumber */
-  WP_LOOKUP_DIR_ETC = (1 << 11),             /*!< ($prefix)/etc/wireplumber */
-  WP_LOOKUP_DIR_PREFIX_SHARE = (1 << 12),    /*!< $prefix/share/wireplumber */
-} WpLookupDirs;
-
-WP_API
-gchar * wp_find_file (WpLookupDirs dirs, const gchar *filename,
-    const char *subdir);
-
-WP_API
-WpIterator * wp_new_files_iterator (WpLookupDirs dirs, const gchar *subdir,
-    const gchar *suffix);
+const char * wp_get_library_api_version (void);
 
 G_END_DECLS
 
