@@ -1522,6 +1522,40 @@ static const luaL_Reg si_adapter_methods[] = {
 
 /* WpPipewireObject */
 
+static void
+on_enum_params_done (WpPipewireObject * pwobj, GAsyncResult * res,
+    GClosure * closure)
+{
+  g_autoptr (GError) error = NULL;
+  GValue val = G_VALUE_INIT;
+  int n_vals = 0;
+  WpIterator *it;
+
+  it = wp_pipewire_object_enum_params_finish (pwobj, res, &error);
+  if (!it) {
+    g_value_init (&val, G_TYPE_STRING);
+    g_value_set_string (&val, error->message);
+    n_vals = 1;
+  }
+  g_clear_pointer (&it, wp_iterator_unref);
+  g_closure_invoke (closure, NULL, n_vals, &val, NULL);
+  g_value_unset (&val);
+  g_closure_invalidate (closure);
+  g_closure_unref (closure);
+}
+
+static int
+pipewire_object_enum_params (lua_State *L)
+{
+  WpPipewireObject *pwobj = wplua_checkobject (L, 1, WP_TYPE_PIPEWIRE_OBJECT);
+  const gchar *id = luaL_checkstring (L, 2);
+  GClosure * closure = wplua_checkclosure (L, 3);
+  g_closure_sink (g_closure_ref (closure));
+  wp_pipewire_object_enum_params (pwobj, id, NULL, NULL,
+      (GAsyncReadyCallback) on_enum_params_done, closure);
+  return 0;
+}
+
 static int
 pipewire_object_iterate_params (lua_State *L)
 {
@@ -1542,6 +1576,7 @@ pipewire_object_set_param (lua_State *L)
 }
 
 static const luaL_Reg pipewire_object_methods[] = {
+  { "enum_params", pipewire_object_enum_params },
   { "iterate_params", pipewire_object_iterate_params },
   { "set_param" , pipewire_object_set_param },
   { "set_params" , pipewire_object_set_param }, /* deprecated, compat only */
@@ -1906,6 +1941,77 @@ static const luaL_Reg json_utils_funcs[] = {
   { NULL, NULL }
 };
 
+/* ProcInfo */
+
+static int
+proc_info_get_pid (lua_State *L)
+{
+  WpProcInfo *pi = wplua_checkboxed (L, 1, WP_TYPE_PROC_INFO);
+  lua_pushinteger (L, wp_proc_info_get_pid (pi));
+  return 1;
+}
+
+static int
+proc_info_get_parent_pid (lua_State *L)
+{
+  WpProcInfo *pi = wplua_checkboxed (L, 1, WP_TYPE_PROC_INFO);
+  lua_pushinteger (L, wp_proc_info_get_parent_pid (pi));
+  return 1;
+}
+
+static int
+proc_info_get_n_args (lua_State *L)
+{
+  WpProcInfo *pi = wplua_checkboxed (L, 1, WP_TYPE_PROC_INFO);
+  lua_pushinteger (L, wp_proc_info_get_n_args (pi));
+  return 1;
+}
+
+static int
+proc_info_get_arg (lua_State *L)
+{
+  WpProcInfo *pi = wplua_checkboxed (L, 1, WP_TYPE_PROC_INFO);
+  guint index = luaL_checkinteger (L, 2);
+  lua_pushstring (L, wp_proc_info_get_arg (pi, index));
+  return 1;
+}
+
+static int
+proc_info_get_cgroup (lua_State *L)
+{
+  WpProcInfo *pi = wplua_checkboxed (L, 1, WP_TYPE_PROC_INFO);
+  lua_pushstring (L, wp_proc_info_get_cgroup (pi));
+  return 1;
+}
+
+static const luaL_Reg proc_info_funcs[] = {
+  { "get_pid", proc_info_get_pid },
+  { "get_parent_pid", proc_info_get_parent_pid },
+  { "get_cgroup", proc_info_get_cgroup },
+  { "get_n_args", proc_info_get_n_args },
+  { "get_arg", proc_info_get_arg },
+  { NULL, NULL }
+};
+
+/* ProcUtils */
+
+static int
+proc_utils_get_proc_info (lua_State *L)
+{
+  guint pid = luaL_checkinteger (L, 1);
+  WpProcInfo *pi = wp_proc_utils_get_proc_info (pid);
+  if (pi)
+    wplua_pushboxed (L, WP_TYPE_PROC_INFO, pi);
+  else
+    lua_pushnil (L);
+  return 1;
+}
+
+static const luaL_Reg proc_utils_funcs[] = {
+  { "get_proc_info", proc_utils_get_proc_info },
+  { NULL, NULL }
+};
+
 /* WpSettings */
 
 static int
@@ -2812,6 +2918,9 @@ wp_lua_scripting_api_init (lua_State *L)
   luaL_newlib (L, json_utils_funcs);
   lua_setglobal (L, "JsonUtils");
 
+  luaL_newlib (L, proc_utils_funcs);
+  lua_setglobal (L, "ProcUtils");
+
   luaL_newlib (L, settings_methods);
   lua_setglobal (L, "WpSettings");
 
@@ -2873,6 +2982,8 @@ wp_lua_scripting_api_init (lua_State *L)
       NULL, transition_methods);
   wplua_register_type_methods (L, WP_TYPE_CONF,
       conf_new, conf_methods);
+  wplua_register_type_methods (L, WP_TYPE_PROC_INFO,
+      NULL, proc_info_funcs);
 
   if (!wplua_load_uri (L, URI_API, &error) ||
       !wplua_pcall (L, 0, 0, &error)) {
